// Customer ID=7729; Build=0x71521; Copyright (c) 2003-2015 Cadence Design Systems, Inc.  ALL RIGHTS RESERVED.
//
// These coded instructions, statements, and computer programs are the
// copyrighted works and confidential proprietary information of
// Cadence Design Systems, Inc.  They may be adapted and modified by bona fide
// purchasers for internal use, but neither the original nor any adapted
// or modified version may be disclosed or distributed to third parties
// in any manner, medium, or form, in whole or in part, without the prior
// written consent of Cadence Design Systems, Inc.



//`define LOG_PEEK_POKE



/**********************************************************************
 *                  -- Module: hifi3_hikey960_dram0 --
 **********************************************************************/
module hifi3_hikey960_dram0 (

	DRam0Data0,
	DRam0Addr0,
	DRam0En0,
	DRam0Wr0,
	DRam0ByteEn0,
	DRam0WrData0,
	DRam0Lock0,
	CLK);

output [63:0] DRam0Data0;  
input  [11:0] DRam0Addr0;  
input         DRam0En0;    
input         DRam0Wr0;    
input  [7:0]  DRam0ByteEn0; 
input  [63:0] DRam0WrData0; 
input         DRam0Lock0;  
input         CLK;         


   parameter file_name = "readmemh.data/dram0.data";

   initial begin
    if ($test$plusargs("dumpvars")) begin
      $dumpvars();
    end
   end

   initial begin
     repeat (1) @(posedge CLK);
     if ($test$plusargs("preloadmems") && (file_name != "<none>")) begin
       $display("%t Preloading memory %m from %s", $time, file_name);
       $readmemh(file_name, mem.dataArray);
     end
   end

    hifi3_hikey960_dram0_xtmemory mem(
                        .CLK                (CLK),
                        .address0           (DRam0Addr0),
                        .enable0            (DRam0En0),
                        .byteEn0            (DRam0ByteEn0),
                        .writeData0         (DRam0WrData0),
                        .writeEnable0       (DRam0Wr0),
                        .xtout0             (DRam0Data0)
                        );




  int mem_beg  =  32'he8058000;
  int mem_end  =  32'he805ffff;

`ifdef MODEL_TECH
  export "DPI-SC" function peek_hifi3_hikey960_dram0;
`else
  export "DPI-C" function peek_hifi3_hikey960_dram0;
`endif
  function int peek_hifi3_hikey960_dram0;
    input int unsigned address;
    inout int unsigned byte_lanes_3210;
    reg [63:0] peek_data;
    int unsigned index;
    int unsigned word;
    word = address[2:2];
    peek_hifi3_hikey960_dram0 = 1;
    if (!(address & 2'h3) && ((address >= mem_beg) && (address <= mem_end))) begin
      peek_hifi3_hikey960_dram0 = 0;
      index = (address - mem_beg) >> 3;
      peek_data = mem.dataArray[index];
      byte_lanes_3210 = (peek_data >> (word * 32)) & 32'hFFFFFFFF;
`ifdef LOG_PEEK_POKE
      $display("%t %m: peek addr=0x%8x data=0x%8x index=0x%8x", $time, address, byte_lanes_3210, index);
`endif
    end
  endfunction


`ifdef MODEL_TECH
  export "DPI-SC" function poke_hifi3_hikey960_dram0;
`else
  export "DPI-C" function poke_hifi3_hikey960_dram0;
`endif
  function int poke_hifi3_hikey960_dram0;
    input int unsigned address;
    input int unsigned byte_lanes_3210;
    reg [63:0] poke_data;
    int unsigned index;
    int unsigned word;
    word = address[2:2];
    poke_hifi3_hikey960_dram0 = 1;
    if (!(address & 2'h3) && ((address >= mem_beg) && (address <= mem_end))) begin
      index = (address - mem_beg) >> 3;
      poke_data = mem.dataArray[index];
           if (word ==  0) poke_data = { poke_data[ 63: 32], byte_lanes_3210                   };
      else                 poke_data = {                     byte_lanes_3210, poke_data[ 31:0] };
      poke_hifi3_hikey960_dram0 = 0;
      mem.dataArray[index] = poke_data;
    end
`ifdef LOG_PEEK_POKE
    $display("%t %m: poke addr=0x%8x data=0x%8x, index=0x%8x", $time, address, byte_lanes_3210, index);
`endif
  endfunction


endmodule




/**********************************************************************
 *             -- Module: hifi3_hikey960_dram0_xtmemory --
 **********************************************************************/
module hifi3_hikey960_dram0_xtmemory (

	CLK,
	enable0,
	address0,
	writeData0,
	writeEnable0,
	byteEn0,
	xtout0);

parameter bytes=8,
	nwords=4096,
	width=64,
	awidth=12;

input               CLK;         
input               enable0;     
input  [awidth-1:0] address0;    
input  [width-1:0]  writeData0;  
input               writeEnable0; 
input  [bytes-1:0]  byteEn0;     
output [width-1:0]  xtout0;      

reg    [width-1:0]  dataArray [0:nwords-1];

reg    [width-1:0]  data0;
reg    [width-1:0]  data0_M;
reg                 enable20;


wire   [awidth-1:0] fixed_address0 = address0;

   //   --- Write --- 
   //   #0 to delay 1 delta cycle so a Read this cycle will return old data
   always @(posedge CLK) #0 begin
      if (enable0 && writeEnable0) begin
            data0_M = dataArray[fixed_address0];
            if (byteEn0[0]) begin
              data0_M[7:0] = writeData0[7:0];
            end
            if (byteEn0[1]) begin
              data0_M[15:8] = writeData0[15:8];
            end
            if (byteEn0[2]) begin
              data0_M[23:16] = writeData0[23:16];
            end
            if (byteEn0[3]) begin
              data0_M[31:24] = writeData0[31:24];
            end
            if (byteEn0[4]) begin
              data0_M[39:32] = writeData0[39:32];
            end
            if (byteEn0[5]) begin
              data0_M[47:40] = writeData0[47:40];
            end
            if (byteEn0[6]) begin
              data0_M[55:48] = writeData0[55:48];
            end
            if (byteEn0[7]) begin
              data0_M[63:56] = writeData0[63:56];
            end
            dataArray[fixed_address0] = data0_M;
      end
   end

   //   --- Read ---
   always @(posedge CLK) begin
      enable20 <= #1 enable0;
      if (enable0) begin
        data0 <= #1 dataArray[fixed_address0];
      end
   end

   assign xtout0 =data0;

endmodule


